APIYNFLAG_YES = "Y"
APIYNFLAG_NO = "N"
APILOGLEVEL_NONE = "N"
APILOGLEVEL_ERROR = "E"
APILOGLEVEL_WARNING = "W"
APILOGLEVEL_DEBUG = "D"
TAPI_COMMODITY_TYPE_NONE = "N"
TAPI_COMMODITY_TYPE_SPOT = "P"
TAPI_COMMODITY_TYPE_FUTURES = "F"
TAPI_COMMODITY_TYPE_OPTION = "O"
TAPI_COMMODITY_TYPE_SPREAD_MONTH = "S"
TAPI_COMMODITY_TYPE_SPREAD_COMMODITY = "M"
TAPI_COMMODITY_TYPE_BUL = "U"
TAPI_COMMODITY_TYPE_BER = "E"
TAPI_COMMODITY_TYPE_STD = "D"
TAPI_COMMODITY_TYPE_STG = "G"
TAPI_COMMODITY_TYPE_PRT = "R"
TAPI_COMMODITY_TYPE_BLT = "L"
TAPI_COMMODITY_TYPE_BRT = "Q"
TAPI_COMMODITY_TYPE_DIRECTFOREX = "X"
TAPI_COMMODITY_TYPE_INDIRECTFOREX = "I"
TAPI_COMMODITY_TYPE_CROSSFOREX = "C"
TAPI_COMMODITY_TYPE_INDEX = "Z"
TAPI_COMMODITY_TYPE_STOCK = "T"
TAPI_COMMODITY_TYPE_SPOT_TRADINGDEFER = "Y"
TAPI_COMMODITY_TYPE_FUTURE_LOCK = "J"
TAPI_COMMODITY_TYPE_EFP = "A"
TAPI_COMMODITY_TYPE_TAS = "B"
TAPI_CALLPUT_FLAG_CALL = "C"
TAPI_CALLPUT_FLAG_PUT = "P"
TAPI_CALLPUT_FLAG_NONE = "N"
TAPI_AUTHTYPE_DIRECT = "1"
TAPI_AUTHTYPE_RELAY = "2"
TAPIERROR_SUCCEED = 0
TAPIERROR_ConnectFail = -1
TAPIERROR_LinkAuthFail = -2
TAPIERROR_HostUnavailable = -3
TAPIERROR_SendDataError = -4
TAPIERROR_TestIDError = -5
TAPIERROR_NotReadyTestNetwork = -6
TAPIERROR_CurTestNotOver = -7
TAPIERROR_NOFrontAvailable = -8
TAPIERROR_DataPathAvaiable = -9
TAPIERROR_RepeatLogin = -10
TAPIERROR_InnerError = -11
TAPIERROR_LastReqNotFinish = -12
TAPIERROR_InputValueError = -13
TAPIERROR_AuthCode_Invalid = -14
TAPIERROR_AuthCode_Expired = -15
TAPIERROR_AuthCode_TypeNotMatch = -16
TAPIERROR_API_NotReady = -17
TAPIERROR_UDP_LISTEN_FAILED = -18
TAPIERROR_UDP_LISTENING = -19
TAPIERROR_NotImplemented = -20
TAPIERROR_CallOneTimeOnly = -21
TAPIERROR_Frequently = -22
TAPIERROR_DataCollect = -23
TAPIERROR_DataLoad = -24
TAPIERROR_NoRelay = -25
TAPIERROR_INPUTERROR_NULL = -10000
TAPIERROR_INPUTERROR_TAPIYNFLAG = -10001
TAPIERROR_INPUTERROR_TAPILOGLEVEL = -10002
TAPIERROR_INPUTERROR_TAPICommodityType = -10003
TAPIERROR_INPUTERROR_TAPICallOrPutFlagType = -10004
TAPIERROR_INPUTERROR_TAPIAccountType = -12001
TAPIERROR_INPUTERROR_TAPIAccountState = -12003
TAPIERROR_INPUTERROR_TAPIAccountFamilyType = -12004
TAPIERROR_INPUTERROR_TAPIOrderTypeType = -12005
TAPIERROR_INPUTERROR_TAPIOrderSourceType = -12006
TAPIERROR_INPUTERROR_TAPITimeInForceType = -12007
TAPIERROR_INPUTERROR_TAPISideType = -12008
TAPIERROR_INPUTERROR_TAPIPositionEffectType = -12009
TAPIERROR_INPUTERROR_TAPIHedgeFlagType = -12010
TAPIERROR_INPUTERROR_TAPIOrderStateType = -12011
TAPIERROR_INPUTERROR_TAPICalculateModeType = -12012
TAPIERROR_INPUTERROR_TAPIMatchSourceType = -12013
TAPIERROR_INPUTERROR_TAPIOpenCloseModeType = -12014
TAPIERROR_INPUTERROR_TAPIFutureAlgType = -12015
TAPIERROR_INPUTERROR_TAPIOptionAlgType = -12016
TAPIERROR_INPUTERROR_TAPIBankAccountLWFlagType = -12017
TAPIERROR_INPUTERROR_TAPIMarginCalculateModeType = -12021
TAPIERROR_INPUTERROR_TAPIOptionMarginCalculateModeType = -12022
TAPIERROR_INPUTERROR_TAPICmbDirectType = -12023
TAPIERROR_INPUTERROR_TAPIDeliveryModeType = -12024
TAPIERROR_INPUTERROR_TAPIContractTypeType = -12025
TAPIERROR_INPUTERROR_TAPITacticsTypeType = -12035
TAPIERROR_INPUTERROR_TAPIORDERACT = -12036
TAPIERROR_INPUTERROR_TAPITriggerConditionType = -12041
TAPIERROR_INPUTERROR_TAPITriggerPriceTypeType = -12042
TAPIERROR_INPUTERROR_TAPITradingStateType = -12043
TAPIERROR_INPUTERROR_TAPIMarketLevelType = -12044
TAPIERROR_INPUTERROR_TAPIOrderQryTypeType = -12045
TAPIERROR_DISCONNECT_CLOSE_INIT = 1
TAPIERROR_DISCONNECT_CLOSE_PASS = 2
TAPIERROR_DISCONNECT_READ_ERROR = 3
TAPIERROR_DISCONNECT_WRITE_ERROR = 4
TAPIERROR_DISCONNECT_BUF_FULL = 5
TAPIERROR_DISCONNECT_IOCP_ERROR = 6
TAPIERROR_DISCONNECT_PARSE_ERROR = 7
TAPIERROR_DISCONNECT_CONNECT_TIMEOUT = 8
TAPIERROR_DISCONNECT_INIT_ERROR = 9
TAPIERROR_DISCONNECT_HAS_CONNECTED = 10
TAPIERROR_DISCONNECT_HAS_EXIT = 11
TAPIERROR_DISCONNECT_TRY_LATER = 12
TAPIERROR_DISCONNECT_HEARTBEAT_FAILED = 13
TAPIERROR_LOGIN = 10001
TAPIERROR_LOGIN_USER = 10002
TAPIERROR_LOGIN_DDA = 10003
TAPIERROR_LOGIN_LICENSE = 10004
TAPIERROR_LOGIN_MODULE = 10005
TAPIERROR_LOGIN_FORCE = 10006
TAPIERROR_LOGIN_STATE = 10007
TAPIERROR_LOGIN_PASS = 10008
TAPIERROR_LOGIN_RIGHT = 10009
TAPIERROR_LOGIN_COUNT = 10010
TAPIERROR_LOGIN_NOTIN_SERVERFLAGUSRES = 10011
TAPIERROR_LOGIN_USER_EXPIRED = 10012
TAPIERROR_LOGIN_NO_ACCOUNT = 10013
TAPIERROR_LOGIN_NO_JGT = 10014
TAPIERROR_LOGIN_ERROR_TIMES = 10015
TAPIERROR_LOGIN_ERROR_AUTHTYPE = 10016
TAPIERROR_LOGIN_ERROR_AUTHEXPIRED = 10017
TAPIERROR_LOGIN_ERROR_PASSWDEXPIRED = 10018
TAPIERROR_LOGIN_ERROR_USERTRUST = 10019
TAPIERROR_LOGIN_ERROR_CLIENTVERSION = 10020
TAPIERROR_LOGIN_ERROR_AUTHCODEINFO = 10021
TAPIERROR_USERINFO_QRY = 10101
TAPIERROR_USERALLRIGHT_QRY = 11001
TAPIERROR_USERALLACCOUNT_QRY = 11501
TAPIERROR_USERPASSWORD_MOD = 11701
TAPIERROR_USERPASSWORD_MOD_SOURCE = 11702
TAPIERROR_USERPASSWORD_MOD_SAME = 11703
TAPIERROR_USERPASSWORD_MOD_COMPLEXITY = 11704
TAPIERROR_ACCOUNTINFO_QRY = 20201
TAPIERROR_TRADENO_QRY = 20701
TAPIERROR_CONTRACTINFO_QRY = 22801
TAPIERROR_SPECIALOPTIONFUTURE_QRY = 22901
TAPIERROR_COMMODITYORDERTYPE_QRY = 25501
TAPIERROR_ORDERTIMEINFORCE_QRY = 25601
TAPIERROR_USER_ORDER_FREQUENCE_QRY = 28901
TAPIERROR_USERSUBMITAUTHTYPE_ERROR = 29591
TAPIERROR_USERSUBMITINFO_EMPTY = 29592
TAPIERROR_USERAUTHKEYVERSION_ERROR = 29593
TAPIERROR_USERSUBMITINFO_PARTY = 29594
TAPIERROR_USERSUBMITINFO_TESTKEY = 29595
TAPIERROR_USERSUBMITINFO_USERNO = 29596
TAPIERROR_ORDERINSERT_ACCOUNT = 60001
TAPIERROR_ORDERINSERT_ACCOUNT_STATE = 60002
TAPIERROR_ORDERINSERT_SIDE_TRADE = 60003
TAPIERROR_ORDERINSERT_OPTIONS_TRADE = 60004
TAPIERROR_ORDERINSERT_COMMODITY_TRADE = 60005
TAPIERROR_ORDERINSERT_OPEN_RIGHT = 60006
TAPIERROR_ORDERINSERT_RISK_CHECK = 60007
TAPIERROR_ORDERINSERT_CONTRACT = 60011
TAPIERROR_ORDERINSERT_TRADEROUTE = 60021
TAPIERROR_ORDERINSERT_POSITIONMAX = 60022
TAPIERROR_ORDER_NOTRADE = 60023
TAPIERROR_ORDER_CLOSE = 60024
TAPIERROR_ORDERINSERT_NOTENOUGHFUND = 60031
TAPIERROR_ORDERINSERT_ORDERTYPE = 60032
TAPIERROR_ORDERINSERT_TIMEINFORCE = 60033
TAPIERROR_ORDERINSERT_NO_TACTICS = 60034
TAPIERROR_ORDERINSERT_POSITION_CANNOT_CLOSE = 60035
TAPIERROR_ORDERINSERT_AUTOCHECK_FAIL = 60036
TAPIERROR_ORDERINSERT_LME_NOTREADY = 60037
TAPIERROR_ORDERINSERT_CLOSEMODE = 60038
TAPIERROR_ORDERINSERT_PARENTNOTENOUGHFUND = 60039
TAPIERROR_SWAP_CONTRACT = 60040
TAPIERROR_ORDERINSERT_PRICE = 60041
TAPIERROR_ORDERINSERT_EFFECT = 60042
TAPIERROR_ORDERINSERT_TARGETCONTRACT = 60043
TAPIERROR_USERNO_NOTHAS_ACCOUNT = 60051
TAPIERROR_UPPERCHANNEL_BROKEN = 60052
TAPIERROR_UPPERCHANNEL_NOT_EXIST = 60053
TAPIERROR_ORDERDELETE_NOT_SYSNO = 60061
TAPIERROR_ORDERDELETE_NOT_STATE = 60062
TAPIERROR_ORDERACTIVE_NOT_STATE = 60063
TAPIERROR_ORDERDELETE_NOT_LAST = 60064
TAPIERROR_ORDERCHECK_NOT_STATE = 60071
TAPIERROR_ORDERCHECK_FAIL = 60072
TAPIERROR_ORDERMODIFY_NOT_STATE = 60081
TAPIERROR_ORDERMODIFY_BACK_INPUT = 60082
TAPIERROR_ORDERINSERT_FEE = 60091
TAPIERROR_ORDERINSERT_MARGIN = 60092
TAPIERROR_ORDER_NO_PERMIT = 60100
TAPIERROR_RSPQUOTE_NO_PERMIT = 60101
TAPIERROR_RSPQUOTE_CHILD_NO_PERMIT = 60102
TAPIERROR_TRADENO_NOT_FIND = 60103
TAPIERROR_ORDER_NO_CLOSE = 60104
TAPIERROR_QRY_QUOTE_NO_PERMIT = 60105
TAPIERROR_EXPIREDATE_NO_PERMIT = 60106
TAPIERROR_CMB_NO_PERMIT = 60107
TAPIERROR_ORDERSERVER_NO_PERMIT = 60108
TAPIERROR_POSITION_CANNOT_EXEC_OR_ABANDON = 60109
TAPIERROR_ORDERCHECK_NO_PERMIT = 60110
TAPIERROR_ORDERMAXOPENVOL_NO_PERMIT = 60111
TAPIERROR_ORDERLIMIT_OPEN_NO_PERMIT = 60112
TAPIERROR_ORDER_SINGLEVOL_NO_PERMIT = 60113
TAPIERROR_ORDER_POSITIONVOL_NO_PERMIT = 60114
TAPIERROR_ORDER_QTY_NO_PERMIT = 60115
TAPIERROR_ORDER_APPLY_NO_PERMIT = 60117
TAPIERROR_ORDER_FREQ_OVERRUN = 60118
TAPIERROR_COMB_NO_SIDEORHEDGE = 60119
TAPIERROR_REQQUOTE_EXITREASONABLPRICE = 60120
TAPIERROR_RSPQUOTE_PRICE = 60121
TAPIERROR_RISKORDER_CANCEL = 60122
TAPIERROR_RSPQUOTE_EFFECT = 60123
TAPIERROR_ORDERINSERT_SIDEMODE = 60124
TAPIERROR_AUTOORDER_MAXCOUNT = 60125
TAPIERROR_SHFEDEEPQUOTE_LIMIT = 60126
TAPIERROR_SHFEDEEPQUOTE_NODATA = 60127
TAPIERROR_SELFMATCH = 60128
TAPIERROR_ERRORORDER_MAXCOUNT = 60129
TAPIERROR_ORDER_UPPERFREQ_OVERRUN = 60130
TAPIERROR_ORDER_FREQUENCY = 61001
TAPIERROR_ORDER_QUERYING = 61002
TAPIERROR_SUBSCRIBEQUOTE_MAX = 72001
TAPIERROR_SUBSCRIBEQUOTE_EXCHANGE_MAX = 72002
TAPIERROR_SUBSCRIBEQUOTE_NO_RIGHT = 72101
TAPIERROR_SUBSCRIBEQUOTE_NO_EXCHANGE_RIGHT = 72102
TAPIERROR_SUBSCRIBEQUOTE_COMMODITY_NOT_EXIST = 72103
TAPIERROR_SUBSCRIBEQUOTE_CONTRACT_MAY_NOT_EXIST = 72104
TAPIERROR_QUOTEFRONT_UNKNOWN_PROTOCOL = 83001
TAPIERROR_QUOTEFRONT_LOGIN_TOTALCOUNT = 83002
TAPIERROR_QUOTEFRONT_SUB_COUNT = 83003
TAPIERROR_ORDER_SEND = 80001
TAPIERROR_DLG_NULL = 80002
TAPIERROR_ORDER_FIELD = 80003
TAPIERROR_TRADE_REJ_BYUPPER = 80004
TAPIERROR_ORDER_FORBIDEXEC = 80005
APIYNFLAG_YES = "Y"
APIYNFLAG_NO = "N"
APILOGLEVEL_ERROR = "1"
APILOGLEVEL_NORMAL = "2"
APILOGLEVEL_DEBUG = "3"
TAPI_COMMODITY_TYPE_NONE = "N"
TAPI_COMMODITY_TYPE_SPOT = "P"
TAPI_COMMODITY_TYPE_FUTURES = "F"
TAPI_COMMODITY_TYPE_OPTION = "O"
TAPI_COMMODITY_TYPE_SPREAD_MONTH = "S"
TAPI_COMMODITY_TYPE_SPREAD_COMMODITY = "M"
TAPI_COMMODITY_TYPE_BUL = "U"
TAPI_COMMODITY_TYPE_BER = "E"
TAPI_COMMODITY_TYPE_STD = "D"
TAPI_COMMODITY_TYPE_STG = "G"
TAPI_COMMODITY_TYPE_PRT = "R"
TAPI_COMMODITY_TYPE_BLT = "L"
TAPI_COMMODITY_TYPE_BRT = "Q"
TAPI_COMMODITY_TYPE_DIRECTFOREX = "X"
TAPI_COMMODITY_TYPE_INDIRECTFOREX = "I"
TAPI_COMMODITY_TYPE_CROSSFOREX = "C"
TAPI_COMMODITY_TYPE_INDEX = "Z"
TAPI_COMMODITY_TYPE_STOCK = "T"
TAPI_COMMODITY_TYPE_SPOT_TRADINGDEFER = "Y"
TAPI_COMMODITY_TYPE_FUTURE_LOCK = "J"
TAPI_COMMODITY_TYPE_SPREAD_C_COMMODITY = "K"
TAPI_COMMODITY_TYPE_EFP = "A"
TAPI_COMMODITY_TYPE_TAS = "B"
TAPI_CALLPUT_FLAG_CALL = "C"
TAPI_CALLPUT_FLAG_PUT = "P"
TAPI_CALLPUT_FLAG_NONE = "N"
TAPI_ACCOUNT_TYPE_PERSON = "P"
TAPI_ACCOUNT_TYPE_ORGANIZATION = "O"
TAPI_ACCOUNT_TYPE_AGENT = "A"
TAPI_ACCOUNT_TYPE_MARGIN = "M"
TAPI_ACCOUNT_TYPE_INTERNAL = "I"
TAPI_ACCOUNT_TYPE_HOUSE = "H"
TAPI_ACCOUNT_TYPE_STOCK = "S"
TAPI_ACCOUNT_TYPE_MARTKET = "R"
TAPI_ACCOUNT_TYPE_GIVEUP = "G"
TAPI_ACCOUNT_TYPE_ERRACCOUNT = "E"
TAPI_RIGHT_ORDER_DEL = 30001
TAPI_RIGHT_ORDER_CHECK = 30002
TAPI_RIGHT_ORDER_MODIFY = 30003
TAPI_RIGHT_ORDER_TRANFER = 30004
TAPI_RIGHT_ORDER_INPUT = 30005
TAPI_RIGHT_ONLY_QRY = 31000
TAPI_RIGHT_ONLY_OPEN = 31001
TAPI_RIGHT_SHFE_QUOTE = 31002
TAPI_RIGHT_ONLY_COVER = 31003
TAPI_USERTYPE_CLIENT = 10000
TAPI_USERTYPE_BROKER = 20000
TAPI_USERTYPE_TRADER = 30000
TAPI_USERTYPE_RISK = 40000
TAPI_USERTYPE_MANAGER = 50000
TAPI_USERTYPE_QUOTE = 60000
TAPI_ACCOUNT_STATE_NORMAL = "N"
TAPI_ACCOUNT_STATE_CANCEL = "C"
TAPI_ACCOUNT_STATE_SLEEP = "S"
TAPI_ACCOUNT_STATE_FROZEN = "F"
TAPI_ACCOUNT_FAMILYTYPE_NOMAL = "N"
TAPI_ACCOUNT_FAMILYTYPE_CHILD = "C"
TAPI_ORDER_TYPE_MARKET = "1"
TAPI_ORDER_TYPE_LIMIT = "2"
TAPI_ORDER_TYPE_STOP_MARKET = "3"
TAPI_ORDER_TYPE_STOP_LIMIT = "4"
TAPI_ORDER_TYPE_OPT_EXEC = "5"
TAPI_ORDER_TYPE_OPT_ABANDON = "6"
TAPI_ORDER_TYPE_REQQUOT = "7"
TAPI_ORDER_TYPE_RSPQUOT = "8"
TAPI_ORDER_TYPE_SWAP = "9"
TAPI_ORDER_TYPE_COMB = "A"
TAPI_ORDER_TYPE_UNCOMB = "B"
TAPI_ORDER_TYPE_PROFIT_MARKET = "C"
TAPI_ORDER_TYPE_PROFIT_LIMIT = "D"
TAPI_ORDER_TYPE_HEDGE = "E"
TAPI_ORDER_TYPE_OPTION_AUTO_CLOSE = "F"
TAPI_ORDER_TYPE_FUTURE_AUTO_CLOSE = "G"
TAPI_ORDER_TYPE_MARKET_POSITION_KEEP = "H"
TAPI_ORDER_TYPE_OPTION_AUTOEXEC_ABAND = "I"
TAPI_ORDER_TYPE_OTC = "K"
TAPI_ORDER_TYPE_ICEBERG = "Z"
TAPI_ORDER_TYPE_GHOST = "Y"
TAPI_ORDER_TYPE_HKEX_AUCTION = "X"
TAPI_ORDER_TYPE_LOCK = "W"
TAPI_ORDER_TYPE_UNLOCK = "V"
TAPI_ORDER_TYPE_ENHANCE = "U"
TAPI_ORDER_TYPE_SPECIAL = "T"
TAPI_ORDER_TYPE_LIMIT_AUCTION = "S"
TAPI_ORDER_SOURCE_SELF_ETRADER = "1"
TAPI_ORDER_SOURCE_PROXY_ETRADER = "2"
TAPI_ORDER_SOURCE_JTRADER = "3"
TAPI_ORDER_SOURCE_MANUAL = "4"
TAPI_ORDER_SOURCE_CARRY = "5"
TAPI_ORDER_SOURCE_PROGRAM = "6"
TAPI_ORDER_SOURCE_DELIVERY = "7"
TAPI_ORDER_SOURCE_ABANDON = "8"
TAPI_ORDER_SOURCE_CHANNEL = "9"
TAPI_ORDER_SOURCE_ESUNNY_API = "A"
TAPI_ORDER_SOURCE_ESV8CLIENT = "B"
TAPI_ORDER_SOURCE_EPOLESTAR = "F"
TAPI_ORDER_SOURCE_ESTAR = "D"
TAPI_ORDER_SOURCE_DCEAFTEREX = "E"
TAPI_ORDER_SOURCE_COMPLIANCE = "C"
TAPI_ORDER_SOURCE_OPTIONHEDGE = "I"
TAPI_ORDER_SOURCE_FUTUREHEDGE = "J"
TAPI_ORDER_SOURCE_OPTIONEXCE = "K"
TAPI_ORDER_SOURCE_ESUNNY_API_UDP = "G"
TAPI_ORDER_SOURCE_ESV85CLIENT = "L"
TAPI_ORDER_SOURCE_ESV95CLIENT = "M"
TAPI_ORDER_SOURCE_EPOLECLOUD = "N"
TAPI_ORDER_SOURCE_ESQUANT = "O"
TAPI_ORDER_SOURCE_BLOOMBERG = "Z"
TAPI_ORDER_TIMEINFORCE_GFD = "0"
TAPI_ORDER_TIMEINFORCE_GTC = "1"
TAPI_ORDER_TIMEINFORCE_GTD = "2"
TAPI_ORDER_TIMEINFORCE_FAK = "3"
TAPI_ORDER_TIMEINFORCE_FOK = "4"
TAPI_SIDE_NONE = "N"
TAPI_SIDE_BUY = "B"
TAPI_SIDE_SELL = "S"
TAPI_SIDE_ALL = "A"
TAPI_PositionEffect_NONE = "N"
TAPI_PositionEffect_OPEN = "O"
TAPI_PositionEffect_COVER = "C"
TAPI_PositionEffect_COVER_TODAY = "T"
TAPI_HEDGEFLAG_NONE = "N"
TAPI_HEDGEFLAG_T = "T"
TAPI_HEDGEFLAG_B = "B"
TAPI_HEDGEFLAG_L = "L"
TAPI_HEDGEFLAG_M = "M"
TAPI_HEDGEFLAG_I = "I"
TAPI_HEDGEFLAG_C = "C"
TAPI_HEDGEFLAG_R = "R"
TAPI_HEDGEFLAG_S = "S"
TAPI_HEDGEFLAG_F = "F"
TAPI_HEDGEFLAG_Z = "Z"
TAPI_ORDER_STATE_SUBMIT = "0"
TAPI_ORDER_STATE_ACCEPT = "1"
TAPI_ORDER_STATE_TRIGGERING = "2"
TAPI_ORDER_STATE_EXCTRIGGERING = "3"
TAPI_ORDER_STATE_QUEUED = "4"
TAPI_ORDER_STATE_PARTFINISHED = "5"
TAPI_ORDER_STATE_FINISHED = "6"
TAPI_ORDER_STATE_CANCELING = "7"
TAPI_ORDER_STATE_MODIFYING = "8"
TAPI_ORDER_STATE_CANCELED = "9"
TAPI_ORDER_STATE_LEFTCANCELED = "A"
TAPI_ORDER_STATE_FAIL = "B"
TAPI_ORDER_STATE_DELETED = "C"
TAPI_ORDER_STATE_SUPPENDED = "D"
TAPI_ORDER_STATE_DELETEDFOREXPIRE = "E"
TAPI_ORDER_STATE_EFFECT = "F"
TAPI_ORDER_STATE_APPLY = "G"
TAPI_ORDER_STATE_SYSTEMDELETED = "H"
TAPI_ORDER_STATE_CHECKING = "I"
TAPI_ORDER_STATE_CLOSECANCELED = "J"
TAPI_CALCULATE_MODE_COMBINE = "0"
TAPI_CALCULATE_MODE_PERCENTAGE = "1"
TAPI_CALCULATE_MODE_QUOTA = "2"
TAPI_CALCULATE_MODE_CHAPERCENTAGE = "3"
TAPI_CALCULATE_MODE_CHAQUOTA = "4"
TAPI_CALCULATE_MODE_DISCOUNT = "5"
TAPI_CALCULATE_MODE_ABSOLUTE = "7"
TAPI_MATCH_SOURCE_ALL = "0"
TAPI_MATCH_SOURCE_SELF_ETRADER = "1"
TAPI_MATCH_SOURCE_PROXY_ETRADER = "2"
TAPI_MATCH_SOURCE_JTRADER = "3"
TAPI_MATCH_SOURCE_MANUAL = "4"
TAPI_MATCH_SOURCE_CARRY = "5"
TAPI_MATCH_SOURCE_PROGRAM = "6"
TAPI_MATCH_SOURCE_DELIVERY = "7"
TAPI_MATCH_SOURCE_ABANDON = "8"
TAPI_MATCH_SOURCE_CHANNEL = "9"
TAPI_MATCH_SOURCE_ESUNNY_API = "A"
TAPI_MATCH_SOURCE_ESV8CLIENT = "B"
TAPI_MATCH_SOURCE_EPOLESTAR = "F"
TAPI_MATCH_SOURCE_ESTAR = "D"
TAPI_MATCH_SOURCE_DCEAFTEREX = "E"
TAPI_MATCH_SOURCE_COMPLIANCE = "C"
TAPI_MATCH_SOURCE_OPTIONHEDGE = "I"
TAPI_MATCH_SOURCE_FUTUREHEDGE = "J"
TAPI_MATCH_SOURCE_OPTIONEXCE = "K"
TAPI_MATCH_SOURCE_BLOOMBERG = "Z"
TAPI_MATCH_SOURCE_GIVEUP = "Y"
TAPI_MATCH_SOURCE_EXERCISE = "X"
TAPI_MATCH_SOURCE_TAKEUP = "W"
TAPI_MATCH_SOURCE_INPUT_DELETE = "#"
TAPI_MATCH_SOURCE_ERRORADJUST = "V"
TAPI_MATCH_SOURCE_BROKER_ETRADER = "U"
TAPI_CLOSE_MODE_NONE = "N"
TAPI_CLOSE_MODE_UNFINISHED = "U"
TAPI_CLOSE_MODE_OPENCOVER = "C"
TAPI_CLOSE_MODE_CLOSETODAY = "T"
TAPI_FUTURES_ALG_ZHUBI = "1"
TAPI_FUTURES_ALG_DINGSHI = "2"
TAPI_OPTION_ALG_FUTURES = "1"
TAPI_OPTION_ALG_OPTION = "2"
TAPI_LWFlag_L = "L"
TAPI_LWFlag_W = "W"
TAPI_DEPOSITCALCULATE_MODE_FEN = "1"
TAPI_DEPOSITCALCULATE_MODE_SUO = "2"
TAPI_CMB_DIRECT_FIRST = "1"
TAPI_CMB_DIRECT_SECOND = "2"
TAPI_DELIVERY_MODE_GOODS = "G"
TAPI_DELIVERY_MODE_CASH = "C"
TAPI_DELIVERY_MODE_EXECUTE = "E"
TAPI_DELIVERY_MODE_ABANDON = "A"
TAPI_DELIVERY_MODE_HKF = "H"
TAPI_DELIVERY_MODE_TAS = "T"
TAPI_DELIVERY_MODE_OPTIONHEDGE = "I"
TAPI_DELIVERY_MODE_FUTUREHEDGE = "J"
TAPI_CONTRACT_TYPE_TRADEQUOTE = "1"
TAPI_CONTRACT_TYPE_QUOTE = "2"
TAPI_ISADD_ONE_YES = "Y"
TAPI_ISADD_ONE_NO = "N"
TAPI_TACTICS_TYPE_NONE = "N"
TAPI_TACTICS_TYPE_READY = "M"
TAPI_TACTICS_TYPE_ATUO = "A"
TAPI_TACTICS_TYPE_CONDITION = "C"
APIORDER_INSERT = "1"
APIORDER_MODIFY = "2"
APIORDER_DELETE = "3"
APIORDER_SUSPEND = "4"
APIORDER_ACTIVATE = "5"
APIORDER_SYSTEM_DELETE = "6"
TAPI_TRIGGER_CONDITION_NONE = "N"
TAPI_TRIGGER_CONDITION_GREAT = "G"
TAPI_TRIGGER_CONDITION_LITTLE = "L"
TAPI_TRIGGER_PRICE_NONE = "N"
TAPI_TRIGGER_PRICE_BUY = "B"
TAPI_TRIGGER_PRICE_SELL = "S"
TAPI_TRIGGER_PRICE_LAST = "L"
TAPI_MARKET_LEVEL_0 = 0
TAPI_MARKET_LEVEL_1 = 1
TAPI_MARKET_LEVEL_5 = 5
TAPI_MARKET_LEVEL_10 = 10
TAPI_TRADE_STATE_BID = "1"
TAPI_TRADE_STATE_MATCH = "2"
TAPI_TRADE_STATE_CONTINUOUS = "3"
TAPI_TRADE_STATE_PAUSED = "4"
TAPI_TRADE_STATE_CLOSE = "5"
TAPI_TRADE_STATE_DEALLAST = "6"
TAPI_TRADE_STATE_GWDISCONNECT = "0"
TAPI_TRADE_STATE_UNKNOWN = "N"
TAPI_TRADE_STATE_INITIALIZE = "I"
TAPI_TRADE_STATE_READY = "R"
TAPI_NOTICE_IGNORE_NONE = 0x00000000
TAPI_NOTICE_IGNORE_ALL = 0xFFFFFFFF
TAPI_NOTICE_IGNORE_FUND = 0x00000001
TAPI_NOTICE_IGNORE_ORDER = 0x00000002
TAPI_NOTICE_IGNORE_FILL = 0x00000004
TAPI_NOTICE_IGNORE_POSITION = 0x00000008
TAPI_NOTICE_IGNORE_CLOSE = 0x00000010
TAPI_NOTICE_IGNORE_POSITIONPROFIT = 0x00000020
TAPI_ORDER_QRY_TYPE_ALL = "A"
TAPI_ORDER_QRY_TYPE_UNENDED = "U"
TAPI_SYSTEM_TYPE_ESUNNY = 1
TAPI_SYSTEM_TYPE_IESUNNY = 2
TAPI_SYSTEM_TYPE_CELLPHONE = 3
TAPI_SYSTEM_TYPE_ETF = 4
TAPI_SYSTEM_TYPE_STOCK = 5
TAPI_LOGINTYPE_NORMAL = 1
TAPI_LOGINTYPE_GMSSL = 2
TAPI_LOGINTYPE_OPENSSL = 3
TAPI_MSG_LEVEL_NORMAL = "1"
TAPI_MSG_LEVEL_IMPORTANT = "2"
TAPI_MSG_LEVEL_IMERGENCY = "3"
TAPI_Msg_TYPE_MANAGER = "1"
TAPI_Msg_TYPE_RISKCONTROL = "2"
TAPI_Msg_QRYTYPE_ALL = "A"
TAPI_Msg_QRYTYPE_VALID = "Y"
TAPI_CASHINOUT_MODE_FEEADJUST = "0"
TAPI_CASHINOUT_MODE_YKADJUST = "1"
TAPI_CASHINOUT_MODE_PLEDGE = "2"
TAPI_CASHINOUT_MODE_INTERESTREVENUE = "3"
TAPI_CASHINOUT_MODE_COLLECTIONCOST = "4"
TAPI_CASHINOUT_MODE_OTHER = "5"
TAPI_CASHINOUT_MODE_COMPANY = "6"
TAPI_CASHINOUT_MODE_AUTOADJUST = "7"
TAPI_CASHINOUT_MODE_STOCKADJSUT = "8"
TAPI_BILL_DATE = "D"
TAPI_BILL_MONTH = "M"
TAPI_BILL_FILE_TXT = "T"
TAPI_BILL_FILE_PDF = "F"
TAPI_AUTHTYPE_DIRECT = "1"
TAPI_AUTHTYPE_RELAY = "2"
TAPI_SENDTYPE_SMS = "S"
TAPI_SENDTYPE_MAIL = "M"
TAPI_SENDTYPE_WEIXIN = "W"
TAPI_ILOGINTYPE_NORMAL = "N"
TAPI_ILOGINTYPE_TEMPORARY = "T"
TAPI_LOGINTYPE_RESETPASSWORD = "R"
TAPI_LOGINTYPE_UNFREEZE = "U"
TAPI_DEVICETYPE_ANDROID = "0"
TAPI_DEVICETYPE_IOS = "1"
TAPI_STRATEGY_COMBINE = "1"
TAPI_STRATEGY_SPLIT = "2"
TAPI_SPOT_LOCK = "3"
TAPI_SPOT_UNLOCK = "4"
TAPI_OPTION_EXERCISE = "5"
TAPI_OPTION_EXERCISE_COMBINE = "6"
TAPI_STRATEGY_C_BULL = "CNSJC"
TAPI_STRATEGY_P_BEAR = "PXSJC"
TAPI_STRATEGY_P_BULL = "PNSJC"
TAPI_STRATEGY_C_BEAR = "CXSJC"
TAPI_STRATEGY_S_STRADDLE = "KS"
TAPI_STRATEGY_S_STRANGLE = "KKS"
TAPI_STRATEGY_ZBD = "ZBD"
TAPI_STRATEGY_ZXJ = "ZXJ"
TAPI_SettleFlag_AutoSettle = "0"
TAPI_SettleFlagh_Manual = "2"
TAPI_ForceModifyPassword_Yes = "Y"
TAPI_ForceModifyPassword_No = "N"
TAPI_ForceModifyPassword_Reset = "R"
TAPI_ForceModifyPassword_SELFUNFREEZE = "U"
TAPI_CERTIFICATE_TYPE_ONE = "1"
TAPI_CERTIFICATE_TYPE_SIX = "6"
TAPI_CERTIFICATE_TYPE_NINE = "9"
TAPI_CERTIFICATE_TYPE_B = "B"
TAPI_CERTIFICATE_TYPE_R = "R"
TAPI_PASSWORD_TYPE_TRADE = "T"
TAPI_PASSWORD_TYPE_RESET = "R"
TAPI_PASSWORD_TYPE_UNFREEZE = "U"
TAPI_OPERATINGSYS_WINDOWS = "1"
TAPI_OPERATINGSYS_LINUX = "2"
TAPI_PERATINGSYS_MAC_IOS = "3"
TAPI_OPERATINGSYS_PHONE_IOS = "4"
TAPI_OPERATINGSYS_ANDROID = "5"
TAPIERROR_SystemType_Error = -35
TAPIERROR_NoNULLAuthCode = -38
TAPIERROR_UserNoNULL = -39
TAPIERROR_UserNoSame = -40
TAPIERROR_UserNoHasOrder = -41
TAPIERROR_BackUpAddressRepeat = -42
TAPIERROR_PasswordType = -43
TAPIERROR_LoginType = -44
TAPIERROR_Identity = -45
TAPIERROR_VertificateCode = -46
TAPIERROR_OnlyEstarSupport = -47
TAPIERROR_ProtocolSupport = -48
TAPIERROR_OrderPricePrecision = -49
TAPIERROR_OrderPriceInvalid = -50
TAPIERROR_INPUTERROR_TAPIClientID = -12053
TAPIERROR_INPUTERROR_TAPIForceModifyPasswordType = -12054
TAPIERROR_INPUTERROR_TAPICertificateType = -12055
TAPIERROR_INPUTERROR_TAPISendTypeType = -12056
TAPIERROR_INPUTERROR_TAPIPasswordType = -12057
TAPIERROR_INPUTERROR_TAPISecondLoginTypeType = -12058
TAPIERROR_LOGIN_ERROR_CLIENTVERSION = 210020
TAPIERROR_LOGIN_ERROR_AUTHCODEINFO = 210021
TAPIERROR_ILOGIN_GATHERINFO_DIRECT = 110023
TAPIERROR_ILOGIN_GATHERINFO_RELAY = 110024
TAPIERROR_ILOGIN_RESET_PASSWORD = 110025
TAPIERROR_ILOGIN_RESET_PASSWORD_EXCEEDED = 110026
TAPIERROR_ILOGIN_OPERATOR_UNALLOWED_RESET = 110027
TAPIERROR_ILOGIN_RESET_PASSWORD_FROZEN = 110028
TAPIERROR_ILOGIN_VERIFYIDENTITY_FAILED = 110029
TAPIERROR_ILOGIN_VERIFYIDENTITY_EXCEED = 110030
TAPIERROR_ILOGIN_LICENSE_EXPIRED = 110031
TAPIERROR_ILOGIN_PROHIBITRESETPASSWORD = 110032
TAPIERROR_ILOGIN_UNFREEZE = 110033
TAPIERROR_ILOGIN_UNFREEZE_EXCEEDED = 110034
TAPIERROR_ILOGIN_PROHIBIT_UNFREEZE = 110035
TAPIERROR_ILOGIN_NONEED_UNFREEZE = 110036
TAPIERROR_USERSUBMITAUTHTYPE_ERROR = 229591
TAPIERROR_USERSUBMITINFO_EMPTY = 229592
TAPIERROR_USERAUTHKEYVERSION_ERROR = 229593
TAPIERROR_USERSUBMITINFO_PARTY = 229594
TAPIERROR_USERSUBMITINFO_TESTKEY = 229595
TAPIERROR_USERSUBMITINFO_USERNO = 229596
TAPIERROR_USERTRUSTDEVICE_ADDLIMITE = 111012
TAPIERROR_UPPERACCOUNTMAPPING_NOTALLOW = 111013
TAPIERROR_CURRENCY_ONLY_ONEBASE = 113001
TAPIERROR_CURRENCY_ONLY_USDHKD = 113002
TAPIERROR_AUTOSWAP_NOTALLOWED = 115001
TAPIERROR_AUTOSWAP_TIMENOTALLOWED = 115002
TAPIERROR_AUTOSWAP_SINGLEQUOTA = 115003
TAPIERROR_AUTOSWAP_SINGLEDAY = 115004
TAPIERROR_AUTOSWAP_ONEDAYQUOTA = 115005
TAPIERROR_AUTOSWAP_COMPANYONEDAYQUOTA = 115006
TAPIERROR_PLEDGE_OUTOFRANGE = 116001
TAPIERROR_ORDERINSERT_PRICE = 260041
TAPIERROR_ORDERINSERT_EFFECT = 260042
TAPIERROR_ORDERINSERT_TARGETCONTRACT = 260043
TAPIERROR_RSPQUOTE_PRICE = 260121
TAPIERROR_RISKORDER_CANCEL = 260122
TAPIERROR_RSPQUOTE_EFFECT = 260123
TAPIERROR_ORDERINSERT_SIDEMODE = 260124
TAPIERROR_AUTOORDER_MAXCOUNT = 260125
TAPIERROR_SHFEDEEPQUOTE_LIMIT = 260126
TAPIERROR_SHFEDEEPQUOTE_NODATA = 260127
TAPIERROR_SELFMATCH = 260128
TAPIERROR_ERRORORDER_MAXCOUNT = 260129
TAPIERROR_ORDER_UPPERFREQ_OVERRUN = 260130
TAPIERROR_ORDERTYPE_LIMIT = 260131
TAPIERROR_IORDERINT_UN_SUPPORT_ERROR = 160007
TAPIERROR_IORDERINSERT_SUBCONTRACT = 160016
TAPIERROR_IORDERINSERT_SPOTBUYQTY = 160017
TAPIERROR_IORDERINSERT_RESERVE_PUT = 160018
TAPIERROR_IORDERINSERT_RESERVE_B_OPEN = 160019
TAPIERROR_IORDERINSERT_RESERVE_S_COVER = 160020
TAPIERROR_ORDER_ETF_POSITIONMAX = 160036
TAPIERROR_ORDER_ETF_BUYPOSITIONMAX = 160037
TAPIERROR_ORDER_ETF_BUYONEDAYPOSITIONMAX = 160038
TAPIERROR_ORDERINSERT_NOTENOUGHSPOT = 160039
TAPIERROR_ORDERINSERT_RESERVE_SPOT = 160040
TAPIERROR_COMMODITY_LOCK = 160043
TAPIERROR_SPOT_ROOT_COMMODITY = 160044
TAPIERROR_SPOT_ROOTCOM_TRADEROUTE = 160045
TAPIERROR_ORDERINSERT_BUYLIMITE = 160059
TAPIERROR_ORDERINSERT_GROUP_BUYLIMITE = 160060
TAPIERROR_ORDERDELETE_NO_TRADE = 160064
TAPIERROR_ORDERINSERT_POSMAX_COM = 160113
TAPIERROR_ORDERINSERT_POSMAX_COM_NET = 160114
TAPIERROR_CLOSE_SPOT_OUT_LOCK = 160124
TAPIERROR_CLOSE_SPOT_OUT_NULL = 160125
TAPIERROR_MATCHINPUT_OCMODE_ERROR = 160146
TAPIERROR_ORDERACTIVATE_GATEWAY_ERROR = 160183
TAPIERROR_TRANSIT_ORDERINSERT_RIGHT = 160191
TAPIERROR_TRANSIT_ORDERINSERT_DISCON = 160192
TAPIERROR_TRANSIT_ORDERINSERT_DISCON_DEST = 160193
TAPIERROR_TRANSIT_ORDERDELETE_DISCON_DEST = 160194
TAPIERROR_TRANSIT_ORDERMODIFY_DISCON_DEST = 160195
TAPIERROR_TRANSIT_ORDER_OPERATOR = 160196
TAPIERROR_ORDER_DISALLOWBUY_ACCOUNT = 160201
TAPIERROR_ORDER_DISALLOWSELL_ACCOUNT = 160202
TAPIERROR_ORDER_DISALLOWBUY_SYSTEM = 160203
TAPIERROR_ORDER_DISALLOWSELL_SYSTEM = 160204
TAPIERROR_ORDER_DIS_SELLOPTION_ACCOUNT = 160205
TAPIERROR_ORDER_DIS_SELLOPTION_SYSTEM = 160206
TAPIERROR_ORDER_CONTRACT_CLOSE = 160207
TAPIERROR_ORDERINSERT_LOANAMOUNT = 160211
TAPIERROR_COMBINE_COMMODITY = 160220
TAPIERROR_COMBINE_CALLORPUT = 160221
TAPIERROR_COMBINE_COMPOSITION = 160222
TAPIERROR_COMBINE_COMPOSITION_QTY = 160223
TAPIERROR_COMBINE_HEDGEFLAG = 160224
TAPIERROR_COMBINE_ORDERSIDE = 160225
TAPIERROR_COMBINE_CONTRACTSIZE = 160226
TAPIERROR_COMBINE_CONTRACTDAYS = 160227
TAPIERROR_COMBINE_STRIKEPRICE = 160228
TAPIERROR_COMBINE_CONTRACT_SAME = 160229
TAPIERROR_ORDERINSERT_UNLOCK_NOE = 160230
TAPIERROR_ORDEROPEN_OPT_SPOT_NOE = 160231
TAPIERROR_ORDERCLOSE_OPT_SPOT_NOE = 160232
TAPIERROR_ORDERINSERT_COVERED_UNVLD = 160233
TAPIERROR_TRADEFRONT_MODULETYPEERR = 190001
TAPIERROR_TRADEFRONT_TOOMANYDATA = 190002
TAPIERROR_TRADEFRONT_NODATA = 190003
TAPIERROR_TRADEFRONT_NOUSER = 190004
TAPIERROR_TRADEFRONT_DISCONNECT_TRADE = 190011
TAPIERROR_TRADEFRONT_DISCONNECT_MANAGE = 190012
TAPIERROR_TRADEFRONT_ACCOUNT = 190021
TAPIERROR_TRADEFRONT_ORDER = 190022
TAPIERROR_TRADEFRONT_FREQUENCY = 190023
TAPIERROR_TRADEFRONT_RUFUSE = 190024
TAPIERROR_TRADEFRONT_SELFMATCH = 190025
TAPIERROR_TRADEFRONT_DELETEFORCE = 190026
TAPIERROR_ORDER_SEND = 280001
TAPIERROR_DLG_NULL = 280002
TAPIERROR_ORDER_FIELD = 280003
TAPIERROR_TRADE_REJ_BYUPPER = 280004
TAPIERROR_ORDER_FORBIDEXEC = 280005
TAPIERROR_DataCollect_SUCCEED = 0
TAPIERROR_IP = -1
TAPIERROR_MAC = -2
TAPIERROR_ComputerName = 3
TAPIERROR_OSVersion = -4
TAPIERROR_HDID = -5
TAPIERROR_CPUSerialID = -6
TAPIERROR_BIOSSerailID = -7
TAPIERROR_SysDiskInfo = -8
TAPIERROR_MacOsSerialID = -9



from collections import defaultdict
from dataclasses import dataclass
from datetime import datetime
from pathlib import Path
from typing import Dict, Tuple, Any, List
import pytz

from vnpy.event import EventEngine
from vnpy.trader.utility import get_folder_path
from vnpy.trader.constant import (
    Exchange,
    Product,
    Direction,
    Status,
    OrderType,
    Offset
)
from vnpy.trader.gateway import BaseGateway
from vnpy.trader.object import (
    CancelRequest,
    OrderRequest,
    SubscribeRequest,
    TickData,
    ContractData,
    OrderData,
    TradeData,
    PositionData,
    AccountData
)

from ..api import MdApi, TdApi


# 委托状态映射
STATUS_ES2VT: Dict[str, Status] = {
    "0": Status.SUBMITTING,
    "1": Status.SUBMITTING,
    "4": Status.NOTTRADED,
    "5": Status.PARTTRADED,
    "6": Status.ALLTRADED,
    "9": Status.CANCELLED,
    "A": Status.CANCELLED,
    "B": Status.REJECTED,
}

# 多空方向映射
DIRECTION_ES2VT: Dict[str, Direction] = {
    "N": Direction.NET,
    "B": Direction.LONG,
    "S": Direction.SHORT,
}
DIRECTION_VT2ES: Dict[Direction, str] = {v: k for k, v in DIRECTION_ES2VT.items()}

# 委托类型映射
ORDERTYPE_ES2VT: Dict[str, OrderType] = {
    "1": OrderType.MARKET,
    "2": OrderType.LIMIT
}
ORDERTYPE_VT2ES = {v: k for k, v in ORDERTYPE_ES2VT.items()}

# 委托有效类型映射
ORDERTIF_VT2ES: Dict[str, str] = {
    "TAPI_ORDER_TIMEINFORCE_GTC": "1",
    "TAPI_ORDER_TIMEINFORCE_FAK": "3",
    "TAPI_ORDER_TIMEINFORCE_FOK": "4"
}

# 开平方向映射
OFFSET_VT2ES: Dict[Offset, str] = {
    Offset.OPEN: "O",
    Offset.CLOSE: "C",
    Offset.CLOSEYESTERDAY: "C",
    Offset.CLOSETODAY: "T",
    Offset.NONE: "N"
}
OFFSET_ES2VT: Dict[str, Offset] = {
    "O": Offset.OPEN,
    "C": Offset.CLOSE,
    "T": Offset.CLOSETODAY,
    "N": Offset.NONE
}

# 交易所映射
EXCHANGE_ES2VT: Dict[str, Exchange] = {
    "CFFEX": Exchange.CFFEX,
    "ZCE": Exchange.CZCE,
    "DCE": Exchange.DCE,
    "SHFE": Exchange.SHFE,
    "INE": Exchange.INE,
    "SGE": Exchange.SGE,
    "CBOT": Exchange.CBOT,
    "CME": Exchange.CME,
    "COMEX": Exchange.COMEX,
    "EUREX": Exchange.EUREX,
    "HKEX": Exchange.HKFE,
    "KRX": Exchange.KRX,
    "LME": Exchange.LME,
    "NYMEX": Exchange.NYMEX,
    "SGX": Exchange.SGX
}
EXCHANGE_VT2ES: Dict[Exchange, str] = {v: k for k, v in EXCHANGE_ES2VT.items()}

# 产品类型映射
PRODUCT_TYPE_ES2VT: Dict[str, Product] = {
    "F": Product.FUTURES,
    "Y": Product.SPOT
}
PRODUCT_TYPE_VT2ES: Dict[Product, str] = {v: k for k, v in PRODUCT_TYPE_ES2VT.items()}

# 行情接口日志级别映射
MDLOGLEVEL_VT2ES: Dict[str, str] = {
    "APILOGLEVEL_NONE": "N",
    "APILOGLEVEL_ERROR": "E",
    "APILOGLEVEL_WARNING": "W",
    "APILOGLEVEL_DEBUG": "D"
}

# 交易接口日志级别映射
TDLOGLEVEL_VT2ES: Dict[str, str] = {
    "APILOGLEVEL_ERROR": "1",
    "APILOGLEVEL_NORMAL": "2",
    "APILOGLEVEL_DEBUG": "3"
}

# 标示类型映射
FLAG_VT2ES: Dict[str, str] = {
    "APIYNFLAG_YES": "Y",
    "APIYNFLAG_NO": "N",
    "TAPI_CALLPUT_FLAG_CALL": "C",
    "TAPI_CALLPUT_FLAG_PUT": "P",
    "TAPI_CALLPUT_FLAG_NONE": "N"
}

# 投机保值类型映射
HEDGETYPE_VT2ES: Dict[str, str] = {
    "TAPI_HEDGEFLAG_NONE": "N",
    "TAPI_HEDGEFLAG_T": "T",
    "TAPI_HEDGEFLAG_B": "B",
    "TAPI_HEDGEFLAG_L": "L",
    "TAPI_HEDGEFLAG_M": "M"
}

# 其他常量
CHINA_TZ = pytz.timezone("Asia/Shanghai")       # 中国时区

# 合约数据全局缓存字典
commodity_infos: Dict[str, "CommodityInfo"] = {}
contract_infos: Dict[Tuple[str, "Exchange"], "ContractInfo"] = {}


class EsunnyGateway(BaseGateway):
    """
    vn.py用于对接易盛柜台的交易接口。
    """

    default_setting: Dict[str, Any] = {
        "行情账号": "",
        "行情密码": "",
        "行情服务器": "",
        "行情端口": 0,
        "行情授权编码": "",
        "交易账号": "",
        "交易密码": "",
        "交易服务器": "",
        "交易端口": 0,
        "交易产品名称": "",
        "交易授权编码": "",
        "交易系统":["内盘","外盘"]
    }

    exchanges: List[str] = list(EXCHANGE_VT2ES.keys())

    def __init__(self, event_engine: EventEngine, gateway_name: str = "ESUNNY") -> None:
        """构造函数"""
        super().__init__(event_engine, gateway_name)

        self.md_api: "QuoteApi" = QuoteApi(self)
        self.td_api: "EsTradeApi" = EsTradeApi(self)

    def connect(self, setting: dict) -> None:
        """连接交易接口"""
        quote_username: str = setting["行情账号"]
        quote_password: str = setting["行情密码"]
        quote_host: str = setting["行情服务器"]
        quote_port: int = setting["行情端口"]
        quote_authcode: str = setting["行情授权编码"]
        trade_username: str = setting["交易账号"]
        trade_password: str = setting["交易密码"]
        trade_host: str = setting["交易服务器"]
        trade_port: int = setting["交易端口"]
        trade_appid: str = setting["交易产品名称"]
        trade_authcode: str = setting["交易授权编码"]
        if setting["交易系统"] == "内盘":
            systype = 1
        else:
            systype = 2

        self.md_api.connect(
            quote_username,
            quote_password,
            quote_host,
            quote_port,
            quote_authcode
        )
        self.td_api.connect(
            trade_username,
            trade_password,
            trade_host,
            trade_port,
            trade_appid,
            trade_authcode,
            systype
        )

    def subscribe(self, req: SubscribeRequest) -> None:
        """订阅行情"""
        self.md_api.subscribe(req)

    def send_order(self, req: OrderRequest) -> str:
        """委托下单"""
        return self.td_api.send_order(req)

    def cancel_order(self, req: CancelRequest) -> None:
        """委托撤单"""
        self.td_api.cancel_order(req)

    def query_account(self) -> None:
        """查询资金"""
        pass

    def query_position(self) -> None:
        """查询持仓"""
        pass

    def close(self) -> None:
        """关闭接口"""
        self.td_api.close()
        self.md_api.close()


class QuoteApi(MdApi):
    """行情API"""

    def __init__(self, gateway: EsunnyGateway) -> None:
        """构造函数"""
        super().__init__()

        self.gateway: EsunnyGateway = gateway
        self.gateway_name: str = gateway.gateway_name

        self.connect_status: bool = False

    def onRspLogin(self, error: int, data: dict) -> None:
        """用户登录请求回报"""
        if error != 0:
            self.gateway.write_log(f"行情服务器登录失败：{error}")
        else:
            self.connect_status = True
            self.gateway.write_log("行情服务器登录成功")

    def onAPIReady(self) -> None:
        """API状态通知回报"""
        self.qryCommodity()

    def onDisconnect(self, reason: int) -> None:
        """服务器连接断开回报"""
        self.connect_status = False
        self.gateway.write_log(f"行情服务器连接断开，原因：{reason}")

    def onRspSubscribeQuote(
        self,
        session: int,
        error: int,
        last: str,
        data: dict
    ) -> None:
        """订阅行情回报"""
        if error != 0:
            self.gateway.write_log(f"订阅行情失败：{error}")
        else:
            self.update_tick(data)

    def onRtnQuote(self, data: dict) -> None:
        """行情数据推送"""
        self.update_tick(data)

    def onRspQryCommodity(
        self,
        session: int,
        error: int,
        last: str,
        data: dict,
    ) -> None:
        """交易品种查询回报"""
        if error != 0:
            self.gateway.write_log("查询交易品种信息失败")
            return

        # 对于金交所的现货，则直接推送合约信息
        if data["CommodityType"] == "Y":
            contract: ContractData = ContractData(
                symbol=data["CommodityNo"],
                exchange=Exchange.SGE,
                name=data["CommodityNo"],
                product=Product.SPOT,
                size=data["ContractSize"],
                pricetick=data["CommodityTickSize"],
                gateway_name=self.gateway.gateway_name
            )
            self.gateway.on_contract(contract)

        # 缓存交易品种的信息
        commodity_info: CommodityInfo = CommodityInfo(
            size=int(data["ContractSize"]),
            pricetick=data["CommodityTickSize"],
            commodity_type=data["CommodityType"],
            commodity_no=data["CommodityNo"],
            exchange_no=data["ExchangeNo"]
        )
        
        if commodity_info.commodity_no == "AU(T+D)":
            commodity_info.size = 1000
        elif commodity_info.commodity_no == "AG(T+D)":
            commodity_info.size = 1

        key: tuple = (data["ExchangeNo"], data["CommodityNo"], data["CommodityType"])
        commodity_infos[key] = commodity_info

        if last == "Y":
            self.gateway.write_log("查询交易品种信息成功")
            self.qryContract({})

    def onRspQryContract(
        self,
        session: int,
        error: int,
        last: str,
        data: dict
    ) -> None:
        """交易合约查询回报"""
        if error != 0:
            self.gateway.write_log("查询交易合约信息失败")
            return

        # 检查是否支持该合约
        exchange: Exchange = EXCHANGE_ES2VT.get(data["ExchangeNo"], None)
        key: tuple = (data["ExchangeNo"], data["CommodityNo"], data["CommodityType"])
        commodity_info: CommodityInfo = commodity_infos.get(key, None)

        if not data or not exchange or not commodity_info:
            return

        # 只处理期货和现货
        if data["CommodityType"] in PRODUCT_TYPE_ES2VT:
            symbol: str = data["CommodityNo"] + data["ContractNo1"]
            product: Product = PRODUCT_TYPE_ES2VT[data["CommodityType"]]

            contract = ContractData(
                symbol=symbol,
                exchange=exchange,
                name=symbol,
                product=product,
                size=commodity_info.size,
                pricetick=commodity_info.pricetick,
                gateway_name=self.gateway.gateway_name
            )
            self.gateway.on_contract(contract)

            contract_info: ContractInfo = ContractInfo(
                name=contract.name,
                contract_no=data["ContractNo1"],
                commodity_type=data["CommodityType"],
                commodity_no=data["CommodityNo"],
                exchange_no=data["ExchangeNo"]
            )
            contract_infos[(contract.symbol, contract.exchange)] = contract_info

        if last == "Y":
            self.gateway.write_log("查询交易合约信息成功")

    def update_tick(self, data: dict) -> None:
        """切片数据类型转换"""
        # 过滤没有时间戳的异常行情数据
        if not data["DateTimeStamp"]:
            return

        if data["ContractNo1"]:
            symbol: str = data["CommodityNo"] + data["ContractNo1"]
        else:
            symbol: str = data["CommodityNo"]
        exchange: Exchange = EXCHANGE_ES2VT[data["ExchangeNo"]]

        tick: TickData = TickData(
            symbol=symbol,
            exchange=exchange,
            datetime=generate_datetime(data["DateTimeStamp"]),
            name=symbol,
            volume=data["QTotalQty"],
            last_price=data["QLastPrice"],
            last_volume=data["QLastQty"],
            limit_up=data["QLimitUpPrice"],
            limit_down=data["QLimitDownPrice"],
            open_price=data["QOpeningPrice"],
            high_price=data["QHighPrice"],
            low_price=data["QLowPrice"],
            pre_close=data["QPreClosingPrice"],
            bid_price_1=data["QBidPrice"][0],
            bid_price_2=data["QBidPrice"][1],
            bid_price_3=data["QBidPrice"][2],
            bid_price_4=data["QBidPrice"][3],
            bid_price_5=data["QBidPrice"][4],
            ask_price_1=data["QAskPrice"][0],
            ask_price_2=data["QAskPrice"][1],
            ask_price_3=data["QAskPrice"][2],
            ask_price_4=data["QAskPrice"][3],
            ask_price_5=data["QAskPrice"][4],
            bid_volume_1=data["QBidQty"][0],
            bid_volume_2=data["QBidQty"][1],
            bid_volume_3=data["QBidQty"][2],
            bid_volume_4=data["QBidQty"][3],
            bid_volume_5=data["QBidQty"][4],
            ask_volume_1=data["QAskQty"][0],
            ask_volume_2=data["QAskQty"][1],
            ask_volume_3=data["QAskQty"][2],
            ask_volume_4=data["QAskQty"][3],
            ask_volume_5=data["QAskQty"][4],
            gateway_name=self.gateway_name,
        )
        self.gateway.on_tick(tick)

    def connect(
        self,
        username: str,
        password: str,
        host: str,
        port: int,
        auth_code: str
    ) -> None:
        """连接服务器"""
        # 禁止重复发起连接，会导致异常崩溃
        if self.connect_status:
            return

        self.init()

        # API基本设置
        path: Path = get_folder_path(self.gateway_name.lower())
        self.setTapQuoteAPIDataPath(str(path).encode("GBK"))
        self.setTapQuoteAPILogLevel(MDLOGLEVEL_VT2ES["APILOGLEVEL_NONE"])

        # 创建API
        req: dict = {
            "AuthCode": auth_code,
            "KeyOperationLogPath": str(path).encode("GBK")
        }
        self.createTapQuoteAPI(req, 0)

        # 设置服务器地址
        self.setHostAddress(host, port)

        # 登录
        data: dict = {
            "UserNo": username,
            "Password": password,
            "ISModifyPassword": FLAG_VT2ES["APIYNFLAG_NO"],
            "ISDDA": FLAG_VT2ES["APIYNFLAG_NO"]
        }
        self.login(data)

    def subscribe(self, req: SubscribeRequest) -> None:
        """订阅行情"""
        contract_info = contract_infos.get((req.symbol, req.exchange), None)
        if not contract_info:
            if req.exchange == Exchange.SGE:
                key: tuple = ("SGE", req.symbol, "Y")
                commodity_info: CommodityInfo = commodity_infos[key]
            else:
                self.gateway.write_log(f"找不到匹配的合约：{req.symbol}和{req.exchange.value}")
                return

        tap_contract: dict = {
            "ExchangeNo": EXCHANGE_VT2ES[req.exchange],
            "CallOrPutFlag1": FLAG_VT2ES["TAPI_CALLPUT_FLAG_NONE"],
            "CallOrPutFlag2": FLAG_VT2ES["TAPI_CALLPUT_FLAG_NONE"]
        }

        if contract_info:
            tap_contract["ContractNo1"] = contract_info.contract_no
            tap_contract["CommodityType"] = contract_info.commodity_type
            tap_contract["CommodityNo"] = contract_info.commodity_no

        else:
            tap_contract["CommodityType"] = commodity_info.commodity_type
            tap_contract["CommodityNo"] = commodity_info.commodity_no

        self.subscribeQuote(tap_contract)

    def close(self) -> None:
        """关闭连接"""
        if self.connect_status:
            self.disconnect()


class EsTradeApi(TdApi):
    """交易API"""

    def __init__(self, gateway: EsunnyGateway) -> None:
        """构造函数"""
        super().__init__()

        self.gateway: EsunnyGateway = gateway
        self.gateway_name: str = gateway.gateway_name

        self.connect_status: bool = False
        self.reqid: int = 0

        self.account_no: str = ""

        self.sys_local_map: Dict[str, str] = {}
        self.local_sys_map: Dict[str, str] = {}
        self.position_details: Dict[Tuple, Dict[str, Dict]] = defaultdict(dict)

    def onConnect(self, userno: str) -> None:
        """服务器连接成功回报"""
        self.connect_status = True
        self.gateway.write_log("交易服务器连接成功")

    def onRspLogin(self, userno: str, error: int, data: dict) -> None:
        """用户登录请求回报"""
        if error != 0:
            self.gateway.write_log(f"交易服务器登录失败，错误码：{error}")
        else:
            self.gateway.write_log("交易服务器登录成功")

    def onAPIReady(self, userno: str) -> None:
        """API状态通知回报"""
        self.gateway.write_log("交易服务器API准备就绪")

    def onDisconnect(self, userno: str, reason: int) -> None:
        """服务器连接断开回报"""
        self.gateway.write_log(f"交易服务器连接断开，原因：{reason}")

    def onRtnFund(self, userno: str, data: dict) -> None:
        """账户资金推送"""
        self.account_no = data["AccountNo"]

        account: AccountData = AccountData(
            accountid=data["AccountNo"],
            balance=data["Balance"],
            frozen=data["Balance"] - data["Available"],
            gateway_name=self.gateway_name
        )
        self.gateway.on_account(account)

    def onRtnPosition(self, userno: str, data: dict) -> None:
        """持仓更新推送"""
        # 生成合约代码
        if data["ExchangeNo"] == "SGE":
            symbol = data["CommodityNo"]
        else:
            symbol = data["CommodityNo"] + data["ContractNo"]

        # 缓存持仓明细
        key: tuple = (symbol, data["MatchSide"])
        details: dict = self.position_details[key]
        details[data["PositionNo"]] = data

        # 汇总持仓明细
        position: PositionData = PositionData(
            symbol=symbol,
            exchange=EXCHANGE_ES2VT[data["ExchangeNo"]],
            direction=DIRECTION_ES2VT[data["MatchSide"]],
            gateway_name=self.gateway_name
        )
        cost: float = 0

        for d in details.values():
            # 汇总持仓量
            position.volume += d["PositionQty"]

            # 汇总昨仓量
            if d["IsHistory"] == "Y":
                position.yd_volume += d["PositionQty"]

            # 累加成本
            cost += d["PositionPrice"] * d["PositionQty"]

        # 计算均价
        if position.volume:
            position.price = cost / position.volume

        self.gateway.on_position(position)

    def onRtnOrder(self, userno: str, requestid: int, data: dict) -> None:
        """委托更新推送"""
        type = ORDERTYPE_ES2VT.get(data["OrderType"], None)
        if not type:
            self.gateway.write_log(f"收到不支持的委托类型{data['OrderType']}，委托号{data['OrderNo']}")
            return

        if data["ErrorCode"] != 0:
            self.gateway.write_log(f"委托下单失败，错误码: {data['ErrorCode']}, 错误信息: {data['ErrorText']}")

        # 过滤委托的临时状态推送
        if data["OrderState"] in {"7", "8"}:
            return

        # 绑定本地和系统委托号映射
        self.local_sys_map[data["RefString"]] = data["OrderNo"]
        self.sys_local_map[data["OrderNo"]] = data["RefString"]

        if data["ExchangeNo"] == "SGE":
            symbol = data["CommodityNo"]
        else:
            symbol = data["CommodityNo"] + data["ContractNo"]

        order: OrderData = OrderData(
            symbol=symbol,
            exchange=EXCHANGE_ES2VT.get(data["ExchangeNo"], None),
            orderid=data["RefString"],
            type=type,
            direction=DIRECTION_ES2VT[data["OrderSide"]],
            offset=OFFSET_ES2VT.get(data["PositionEffect"], Offset.NONE),
            price=data["OrderPrice"],
            volume=data["OrderQty"],
            traded=data["OrderMatchQty"],
            status=STATUS_ES2VT.get(data["OrderState"], Status.SUBMITTING),
            datetime=generate_datetime(data["OrderInsertTime"]),
            gateway_name=self.gateway_name
        )
        self.gateway.on_order(order)

    def onRtnFill(self, userno: str, data: dict) -> None:
        """成交数据推送"""
        orderid: str = self.sys_local_map[data["OrderNo"]]

        if data["ExchangeNo"] == "SGE":
            symbol = data["CommodityNo"]
        else:
            symbol = data["CommodityNo"] + data["ContractNo"]

        trade: TradeData = TradeData(
            symbol=symbol,
            exchange=EXCHANGE_ES2VT.get(data["ExchangeNo"], None),
            orderid=orderid,
            tradeid=data["MatchNo"],
            direction=DIRECTION_ES2VT[data["MatchSide"]],
            price=data["MatchPrice"],
            volume=data["MatchQty"],
            datetime=generate_datetime(data["MatchDateTime"]),
            gateway_name=self.gateway_name
        )
        self.gateway.on_trade(trade)

    def connect(
        self,
        username: str,
        password: str,
        host: str,
        port: int,
        appid: str,
        auth_code: str,
        systype: int
    ) -> None:
        """连接交易接口"""
        # 禁止重复发起连接，会导致异常崩溃
        if self.connect_status:
            return

        self.userno = username

        self.init()

        # 创建API
        self.createEsTradeAPI(0)

        # API基本设置
        path: Path = get_folder_path(self.gateway_name.lower())
        self.setEsTradeAPIDataPath(str(path).encode("GBK"))
        self.setEsTradeAPILogLevel(TDLOGLEVEL_VT2ES["APILOGLEVEL_ERROR"])

        # 设置用户信息
        user_data: dict = {
            "SystemType": systype,             
            "UserNo": self.userno,
            "LoginIP": host,
            "LoginPort": port,
            "LoginType": 1              # 普通登录
        }
        self.setUserInfo(user_data)

        # 登录
        data: dict = {
            "UserNo": self.userno,
            "Password": password,
            "AuthCode": auth_code,
            "AppID": appid,
            "UserType": 10000,
            "ISModifyPassword": FLAG_VT2ES["APIYNFLAG_NO"],
            "ISDDA": FLAG_VT2ES["APIYNFLAG_NO"],
            "LoginIP": host
        }
        self.startUser(username, data)

    def send_order(self, req: OrderRequest) -> str:
        """委托下单"""
        # 验证是否能找到匹配的合约
        valid = True

        if req.exchange == Exchange.SGE:
            key: tuple = ("SGE", req.symbol, "Y")
            commodity_info: CommodityInfo = commodity_infos.get(key, None)
            if not commodity_info:
                valid = False
        else:
            key: tuple = (req.symbol, req.exchange)
            contract_info: ContractInfo = contract_infos.get(key, None)
            if not contract_info:
                valid = False

        if not valid:
            self.gateway.write_log(f"找不到匹配的合约：{req.symbol}和{req.exchange.value}")
            return

        # 检查委托类型是否支持
        if req.type not in ORDERTYPE_VT2ES:
            self.gateway.write_log(f"不支持的委托类型: {req.type.value}")
            return ""

        # 生成委托号
        self.reqid += 1
        prefix: str = datetime.now().strftime("%Y%m%d_%H%M%S_")
        suffix: str = str(self.reqid).rjust(6, "0")
        orderid: str = f"{prefix}_{suffix}"

        order_req: dict = {
            "AccountNo": self.account_no,
            "OrderType": ORDERTYPE_VT2ES.get(req.type, ""),
            "TimeInForce": ORDERTIF_VT2ES["TAPI_ORDER_TIMEINFORCE_GTC"],
            "PositionEffect": OFFSET_VT2ES.get(req.offset, ""),
            "PositionEffect2": OFFSET_VT2ES[Offset.NONE],
            "OrderSide": DIRECTION_VT2ES.get(req.direction, ""),
            "OrderPrice": req.price,
            "OrderQty": int(req.volume),
            "RefInt": self.reqid,
            "RefString": orderid,
            "CallOrPutFlag": "N",
            "CallOrPutFlag2": "N",
            "IsRiskOrder": FLAG_VT2ES["APIYNFLAG_NO"],
            "AddOneIsValid": FLAG_VT2ES["APIYNFLAG_NO"],
            "FutureAutoCloseFlag": FLAG_VT2ES["APIYNFLAG_NO"]
        }

        if req.exchange == Exchange.SGE:
            order_req["ExchangeNo"] = commodity_info.exchange_no
            order_req["ContractNo"] = commodity_info.commodity_no
            order_req["CommodityType"] = commodity_info.commodity_type
            order_req["CommodityNo"] = commodity_info.commodity_no
            order_req["HedgeFlag"] = HEDGETYPE_VT2ES["TAPI_HEDGEFLAG_NONE"]
            order_req["HedgeFlag2"] = HEDGETYPE_VT2ES["TAPI_HEDGEFLAG_NONE"]
        else:
            order_req["ExchangeNo"] = contract_info.exchange_no
            order_req["ContractNo"] = contract_info.contract_no
            order_req["CommodityType"] = contract_info.commodity_type
            order_req["CommodityNo"] = contract_info.commodity_no
            order_req["HedgeFlag"] = HEDGETYPE_VT2ES["TAPI_HEDGEFLAG_T"]
            order_req["HedgeFlag2"] = HEDGETYPE_VT2ES["TAPI_HEDGEFLAG_T"]

        self.insertOrder(self.userno, order_req, self.reqid)

        order: OrderData = req.create_order_data(orderid, self.gateway_name)
        self.gateway.on_order(order)

        return order.vt_orderid

    def cancel_order(self, req: CancelRequest) -> None:
        """委托撤单"""
        order_no: str = self.local_sys_map.get(req.orderid, "")
        if not order_no:
            self.gateway.write_log(f"撤单失败，找不到{req.orderid}对应的系统委托号")
            return

        cancel_req: dict = {"OrderNo": order_no}

        self.reqid += 1
        self.cancelOrder(self.userno, cancel_req, self.reqid)

    def close(self) -> None:
        """关闭连接"""
        if self.connect_status:
            self.stopUser(self.userno)


def generate_datetime(timestamp: str) -> datetime:
    """生成时间戳"""
    if "-" in timestamp:
        if "." in timestamp:
            dt = datetime.strptime(timestamp, "%Y-%m-%d %H:%M:%S.%f")
        else:
            dt = datetime.strptime(timestamp, "%Y-%m-%d %H:%M:%S")
    else:
        dt = datetime.strptime(timestamp, "%y%m%d%H%M%S.%f")

    dt = CHINA_TZ.localize(dt)
    return dt


@dataclass
class ContractInfo:
    """储存合约信息"""
    name: str
    contract_no: str
    exchange_no: str
    commodity_type: int
    commodity_no: str


@dataclass
class CommodityInfo:
    """储存品种信息"""
    size: int
    pricetick: float
    exchange_no: str
    commodity_type: int
    commodity_no: str
